package com.simuwang.daq.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Pair;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.simuwang.base.common.conts.EmailTypeConst;
import com.simuwang.base.common.util.ExcelUtil;
import com.simuwang.base.common.util.StringUtil;
import com.simuwang.base.pojo.dto.EmailContentInfoDTO;
import com.simuwang.base.pojo.dto.EmailFundNavDTO;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.stereotype.Component;

import java.io.File;
import java.util.List;
import java.util.Map;

/**
 * @author mozuwen
 * @date 2024-09-04
 * @description 净值邮件解析器
 */
@Component
public class NavEmailParser extends AbstractEmailParser {

    @Override
    public boolean isSupport(Integer emailType) {
        return EmailTypeConst.NAV_EMAIL_TYPE.equals(emailType);
    }

    @Override
    public List<EmailFundNavDTO> parse(EmailContentInfoDTO emailContentInfoDTO, Map<String, List<String>> emailFieldMap) {
        List<EmailFundNavDTO> emailFundNavDTOList = CollUtil.newArrayList();
        String emailContent = emailContentInfoDTO.getEmailContent();
        // 1.解析邮件正文
        if (StrUtil.isNotBlank(emailContent)) {
            emailFundNavDTOList = parseEmailContent(emailContent, emailFieldMap);
        }
        // 2.解析邮件excel附件
        if (StrUtil.isNotBlank(emailContentInfoDTO.getFilePath()) && ExcelUtil.isExcel(emailContentInfoDTO.getFileName())) {
            List<EmailFundNavDTO> fundNavDTOList = parseExcelFile(emailContentInfoDTO.getFilePath(), emailFieldMap);
            emailFundNavDTOList.addAll(fundNavDTOList);
        }
        return emailFundNavDTOList;
    }

    /**
     * 解析邮件excel附件
     *
     * @param filePath      邮件excel附件地址
     * @param emailFieldMap 邮件字段识别规则映射表
     * @return 解析到的净值数据
     */
    private List<EmailFundNavDTO> parseExcelFile(String filePath, Map<String, List<String>> emailFieldMap) {
        List<EmailFundNavDTO> fundNavDTOList = CollUtil.newArrayList();
        File file = new File(filePath);
        Sheet sheet = ExcelUtil.getSheet(file, 0);
        Map<String, Pair<Integer, Integer>> fieldPositionMap = getFieldPosition(sheet, emailFieldMap);
        if (MapUtil.isEmpty(fieldPositionMap)) {

        }
        return fundNavDTOList;
    }


    private Map<String, Pair<Integer, Integer>> getFieldPosition(Sheet sheet, Map<String, List<String>> emailFieldMap) {
        Map<String, Pair<Integer, Integer>> fieldPositionMap = MapUtil.newHashMap(8);
        int lastRowNum = sheet.getLastRowNum();
        for (int rowNum = 0; rowNum < lastRowNum; rowNum++) {
            Row sheetRow = sheet.getRow(rowNum);
            int lastCellNum = sheetRow.getLastCellNum();
            for (int cellNum = 0; cellNum < lastCellNum; cellNum++) {
                String cellValue = sheetRow.getCell(cellNum).getStringCellValue();
                String field = fieldMatch(cellValue, emailFieldMap);
                if (StrUtil.isNotBlank(field) && !fieldPositionMap.containsKey(field)) {
                    fieldPositionMap.put(field, new Pair<>(rowNum, cellNum));
                }
            }
        }
        return fieldPositionMap;
    }

    public String fieldMatch(String cellValue, Map<String, List<String>> emailFieldMap) {
        if (StrUtil.isBlank(cellValue)) {
            return null;
        }
        String newCellValue = StringUtil.retainChineseCharacters(cellValue);
        for (Map.Entry<String, List<String>> fieldEntry : emailFieldMap.entrySet()) {
            List<String> fieldList = fieldEntry.getValue();
            for (String field : fieldList) {
                if (newCellValue.equals(field)) {
                    return fieldEntry.getKey();
                }
            }
        }
        return null;
    }

    /**
     * 解析邮件正文
     *
     * @param emailContent  邮件excel附件地址
     * @param emailFieldMap 邮件字段识别规则映射表
     * @return 解析到的净值数据
     */
    private List<EmailFundNavDTO> parseEmailContent(String emailContent, Map<String, List<String>> emailFieldMap) {
        List<EmailFundNavDTO> fundNavDTOList = CollUtil.newArrayList();
        Document doc = Jsoup.parse(emailContent);
        Element table = doc.select("table").first();
        Elements rows = table.select("tr");
        for (Element row : rows) {
            Elements cells = row.select("td");
            int cellSize = cells.size();
            // todo 后续补充
        }
        return fundNavDTOList;
    }

}
